Module: Lutaml::Model::DeepDupable
- Included in:
- Attribute, Mapping, MappingRule, Sequence
- Defined in:
- lib/lutaml/model/deep_dupable.rb
Overview
Protocol module for objects that support deep duplication.
Include this module in any class that needs custom deep-copy semantics beyond Ruby’s default ‘dup`. Used by Utils.deep_dup to dispatch correctly via type checking instead of respond_to?.
Instance Method Summary collapse
Instance Method Details
#deep_dup ⇒ Object
11 12 13 |
# File 'lib/lutaml/model/deep_dupable.rb', line 11 def deep_dup raise NotImplementedError, "#{self.class} must implement deep_dup" end |