Module: Lutaml::Toml
- Defined in:
- lib/lutaml/toml.rb,
lib/lutaml/toml/adapter.rb,
lib/lutaml/toml/adapter/mapping.rb,
lib/lutaml/toml/adapter/document.rb,
lib/lutaml/toml/type/serializers.rb,
lib/lutaml/toml/adapter/transform.rb,
lib/lutaml/toml/adapter/mapping_rule.rb,
lib/lutaml/toml/adapter/tomlib_adapter.rb,
lib/lutaml/toml/adapter/toml_rb_adapter.rb
Defined Under Namespace
Modules: Adapter, Type Classes: Error
Class Method Summary collapse
-
.const_missing(name) ⇒ Object
Convenience aliases for common classes at the module level Allows Lutaml::Toml::Mapping to resolve to Lutaml::Toml::Adapter::Mapping.
Class Method Details
.const_missing(name) ⇒ Object
Convenience aliases for common classes at the module level Allows Lutaml::Toml::Mapping to resolve to Lutaml::Toml::Adapter::Mapping
16 17 18 19 20 21 22 |
# File 'lib/lutaml/toml.rb', line 16 def self.const_missing(name) if Adapter.const_defined?(name, false) Adapter.const_get(name, false) else super end end |