Module: Lutaml::Xml::DataModel

Defined in:
lib/lutaml/xml/data_model.rb

Overview

Pure data classes for XML intermediate representation.

These classes represent XML structure without serialization logic, allowing transformation to produce XML data that can be serialized by different adapters (Nokogiri, Ox, Oga).

Defined Under Namespace

Classes: XmlAttribute, XmlElement, XmlProcessingInstruction