Class: Lutaml::Xml::ProcessingInstructionMapping

Inherits:
Object
  • Object
show all
Defined in:
lib/lutaml/xml/mapping.rb

Overview

Represents a processing instruction mapping in the XML DSL.

Maps a PI target (e.g., "rfc") to a model attribute. The attribute should be a Hash where keys are PI parameter names and values are their string values.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(target, to) ⇒ ProcessingInstructionMapping

Returns a new instance of ProcessingInstructionMapping.



1622
1623
1624
1625
# File 'lib/lutaml/xml/mapping.rb', line 1622

def initialize(target, to)
  @target = target
  @to = to
end

Instance Attribute Details

#targetObject (readonly)

Returns the value of attribute target.



1620
1621
1622
# File 'lib/lutaml/xml/mapping.rb', line 1620

def target
  @target
end

#toObject (readonly)

Returns the value of attribute to.



1620
1621
1622
# File 'lib/lutaml/xml/mapping.rb', line 1620

def to
  @to
end

Instance Method Details

#dupObject



1627
1628
1629
# File 'lib/lutaml/xml/mapping.rb', line 1627

def dup
  self.class.new(@target.dup, @to)
end