Class: Lutaml::Model::Attribute
- Inherits:
-
Object
- Object
- Lutaml::Model::Attribute
- Includes:
- CollectionHandler, DeepDupable
- Defined in:
- lib/lutaml/model/attribute.rb
Constant Summary collapse
- ALLOWED_OPTIONS =
%i[ raw default delegate collection values pattern transform choice sequence method method_name polymorphic polymorphic_class initialize_empty validations required ref_model_class ref_key_attribute xsd_type ].freeze
- MODEL_STRINGS =
[ Lutaml::Model::Type::String, "String", :string, ].freeze
- CHILD_PROPAGATION_KEYS =
Keys that are safe to propagate from parent to child deserialization. Parent-internal keys (namespace_uri, resolved_type, converted) are intentionally excluded — children must derive their own context.
%i[ lutaml_parent lutaml_root default_namespace import_declaration_plan polymorphic collection render_empty render_nil cdata ].freeze
- WARN_ON_OVERRIDE =
Methods where accidental override is likely to cause issues All names are allowed - this list only controls which ones get a warning Format-specific serialization methods (to_xml, to_json, etc.) are pushed by each format plugin at load time via format_specific_warn_names.
%i[ # Ruby core - overriding breaks fundamental behavior hash object_id class send method # Object lifecycle - overriding without super breaks things initialize # Serialization methods - overriding breaks serialization to_format # Internal helpers - overriding breaks internal logic attr_value attribute_exist? key_exist? key_value using_default? using_default_for value_set_for method_missing respond_to_missing? ].freeze
- EMPTY_TRANSFORM_HASH =
Performance: Frozen empty hash to reduce allocations
{}.freeze
- EMPTY_VALUES_ARRAY =
Performance: Frozen empty array to reduce allocations
[].freeze
Instance Attribute Summary collapse
-
#name ⇒ Object
readonly
Returns the value of attribute name.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
- #validations ⇒ Object readonly
Class Method Summary collapse
- .cast_from_string!(type) ⇒ Object
- .cast_from_symbol!(type) ⇒ Object
- .cast_type!(type) ⇒ Object
- .format_specific_warn_names ⇒ Object
- .warn_on_override_names ⇒ Object
Instance Method Summary collapse
- #cast(value, format, register, options = {}) ⇒ Object
- #cast_element(value, register) ⇒ Object
- #cast_type!(type) ⇒ Object
- #cast_value(value, register) ⇒ Object
-
#choice ⇒ Object
min_collection_zero? is provided by CollectionHandler module.
- #deep_dup ⇒ Object
- #default(register = Lutaml::Model::Config.default_register, instance_object = nil) ⇒ Object
- #default_set?(register, instance_object = nil) ⇒ Boolean
-
#default_type_context ⇒ Object
Performance: Cache default type context lookup.
- #default_value(register, instance_object = nil) ⇒ Object
- #delegate ⇒ Object
- #derived? ⇒ Boolean
- #enum? ⇒ Boolean
- #enum_values ⇒ Object
-
#execute_validations!(value) ⇒ Object
execute custom validations on the attribute value i.e presence: true, numericality: true, etc.
-
#extract_register(_context_or_register) ⇒ Object
Extract the Register from the context_or_register argument.
- #hash_type? ⇒ Boolean
- #immutable_value?(value) ⇒ Boolean
-
#initialize(name, type, options = {}) ⇒ Attribute
constructor
A new instance of Attribute.
- #initialize_empty? ⇒ Boolean
- #method_name ⇒ Object
- #model_instance?(value) ⇒ Boolean
-
#normalize_context(context_or_register) ⇒ Object
Normalize register/context to TypeContext for resolution Performance: Optimized to reduce allocations in hot path.
- #pattern ⇒ Object
- #polymorphic? ⇒ Boolean
- #process_options! ⇒ Object
-
#raw? ⇒ Boolean
Collection methods are provided by CollectionHandler module.
- #reference_key(value) ⇒ Object
- #required_value_set?(value) ⇒ Boolean
-
#resolver ⇒ Object
Cache the resolver reference — GlobalContext.resolver is a singleton that never changes during the process lifetime.
-
#sequenced_appearance_count(element_order, mapped_name, current_index) ⇒ Object
resolved_collection is provided by CollectionHandler module.
- #serializable?(register) ⇒ Boolean
-
#serializable_type?(register = nil) ⇒ Boolean
Cached version of serializable? check for hot deserialization path Called millions of times in resolve_rule_names_with_type and value_for_rule.
- #serialize(value, format, register, options = {}) ⇒ Object
- #setter ⇒ Object
- #transform ⇒ Object
- #transform_export_method ⇒ Object
- #transform_import_method ⇒ Object
- #type(context_or_register = nil) ⇒ Object
-
#type_namespace_class(register = nil) ⇒ Class?
Get namespace class from Type::Value or Model class.
-
#type_namespace_prefix(register = nil) ⇒ String?
Get namespace prefix from type.
-
#type_namespace_uri(register = nil) ⇒ String?
Get namespace URI from type.
-
#type_with_namespace(register, namespace_uri = nil) ⇒ Class?
Get type with namespace-aware resolution.
- #unresolved_type ⇒ Object
- #valid_collection!(value, caller) ⇒ Object
- #valid_pattern!(value, resolved_type) ⇒ Object
- #valid_value!(value) ⇒ Object
- #valid_value?(value) ⇒ Boolean
- #validate_choice_content!(elements) ⇒ Object
- #validate_collection_range ⇒ Object
- #validate_polymorphic(value, resolved_type) ⇒ Object
- #validate_polymorphic!(value, resolved_type) ⇒ Object
- #validate_range!(range) ⇒ Object
-
#validate_value!(value, register, resolver = nil) ⇒ Object
Check if the value to be assigned is valid for the attribute.
Methods included from CollectionHandler
#build_collection, #collection, #collection?, #collection_class, #collection_instance?, #custom_collection?, #min_collection_zero?, #resolved_collection, #singular?
Constructor Details
#initialize(name, type, options = {}) ⇒ Attribute
Returns a new instance of Attribute.
108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 |
# File 'lib/lutaml/model/attribute.rb', line 108 def initialize(name, type, = {}) skip_validation = .fetch(:skip_validation, false) unless skip_validation validate_name!( name, reserved_methods: Lutaml::Model::Serializable.instance_methods ) end @name = name @options = .except(:skip_validation) validate_presence!(type) unless skip_validation @type = type @unresolved_type = type unless skip_validation end |
Instance Attribute Details
#name ⇒ Object (readonly)
Returns the value of attribute name.
6 7 8 |
# File 'lib/lutaml/model/attribute.rb', line 6 def name @name end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
6 7 8 |
# File 'lib/lutaml/model/attribute.rb', line 6 def @options end |
#validations ⇒ Object
277 278 279 |
# File 'lib/lutaml/model/attribute.rb', line 277 def validations @options[:validations] end |
Class Method Details
.cast_from_string!(type) ⇒ Object
102 103 104 105 106 |
# File 'lib/lutaml/model/attribute.rb', line 102 def self.cast_from_string!(type) Type.const_get(type) rescue NameError raise ArgumentError, "Unknown Lutaml::Model::Type: #{type}" end |
.cast_from_symbol!(type) ⇒ Object
96 97 98 99 100 |
# File 'lib/lutaml/model/attribute.rb', line 96 def self.cast_from_symbol!(type) Type.lookup(type) rescue UnknownTypeError raise ArgumentError, "Unknown Lutaml::Model::Type: #{type}" end |
.cast_type!(type) ⇒ Object
86 87 88 89 90 91 92 93 94 |
# File 'lib/lutaml/model/attribute.rb', line 86 def self.cast_type!(type) case type when Symbol then cast_from_symbol!(type) when String then cast_from_string!(type) when Class then type else raise ArgumentError, "Unknown Lutaml::Model::Type: #{type}" end end |
.format_specific_warn_names ⇒ Object
78 79 80 |
# File 'lib/lutaml/model/attribute.rb', line 78 def self.format_specific_warn_names @format_specific_warn_names end |
.warn_on_override_names ⇒ Object
82 83 84 |
# File 'lib/lutaml/model/attribute.rb', line 82 def self.warn_on_override_names WARN_ON_OVERRIDE + @format_specific_warn_names end |
Instance Method Details
#cast(value, format, register, options = {}) ⇒ Object
582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 |
# File 'lib/lutaml/model/attribute.rb', line 582 def cast(value, format, register, = {}) # Namespace-aware type resolution: use type_with_namespace if namespace_uri provided namespace_uri = [:namespace_uri] resolved_type = if [:resolved_type] [:resolved_type] elsif register && namespace_uri type_with_namespace(register, namespace_uri) else type(register) end if collection_instance?(value) || value.is_a?(Array) return build_collection(value.map do |v| cast(v, format, register, .merge(resolved_type: resolved_type, converted: true)) end) end return value if already_serialized?(resolved_type, value) # Special handling for Reference types - pass the metadata # Check @options[:ref_model_class] which is set when type is { ref: [...] } if @options[:ref_model_class] && resolved_type == Lutaml::Model::Type::Reference return resolved_type.(value, @options[:ref_model_class], @options[:ref_key_attribute]) end # Fast path for Type::Value subclasses (String, Integer, Boolean, etc.) # These are never Serializable, so skip expensive can_serialize? and needs_conversion? checks # Skip if type has custom from_xml/from_json methods (defined on the class itself, not inherited) if resolved_type.is_a?(Class) && resolved_type < Lutaml::Model::Type::Value has_custom_from_xml = resolved_type.method(:from_xml).owner != Lutaml::Model::Type::Value has_custom_from_json = resolved_type.method(:from_json).owner != Lutaml::Model::Type::Value return resolved_type.cast(value) unless has_custom_from_xml || has_custom_from_json end klass = resolve_polymorphic_class(resolved_type, value, ) if can_serialize?(klass, value, format, ) propagated = .slice(*CHILD_PROPAGATION_KEYS) klass.apply_mappings(value, format, propagated.merge(register: register)) elsif needs_conversion?(klass, value) klass.send(:"from_#{format}", value) else # No need to use register#get_class, # can_serialize? method already checks if type is Serializable or not. Type.lookup(klass).cast(value) end end |
#cast_element(value, register) ⇒ Object
299 300 301 302 303 304 305 306 307 308 309 310 311 312 |
# File 'lib/lutaml/model/attribute.rb', line 299 def cast_element(value, register) resolved_type = type(register) return resolved_type.new(value) if value.is_a?(::Hash) && !hash_type? # Special handling for Reference types - pass the metadata if unresolved_type == Lutaml::Model::Type::Reference return resolved_type.(value, @options[:ref_model_class], @options[:ref_key_attribute]) end validate_attr_type!(resolved_type) resolved_type.cast(value) end |
#cast_type!(type) ⇒ Object
281 282 283 |
# File 'lib/lutaml/model/attribute.rb', line 281 def cast_type!(type) self.class.cast_type!(type) end |
#cast_value(value, register) ⇒ Object
285 286 287 288 289 |
# File 'lib/lutaml/model/attribute.rb', line 285 def cast_value(value, register) return cast_element(value, register) unless collection_instance?(value) build_collection(value.map { |v| cast_element(v, register) }) end |
#choice ⇒ Object
min_collection_zero? is provided by CollectionHandler module
672 673 674 |
# File 'lib/lutaml/model/attribute.rb', line 672 def choice @options[:choice] end |
#deep_dup ⇒ Object
683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 |
# File 'lib/lutaml/model/attribute.rb', line 683 def deep_dup # Don't deep_dup the entire options hash because: # 1. Lambdas/Procs (like :default) should not be duplicated # 2. Classes (like :collection class) are immutable # 3. Deep dupping creates circular references when lambdas close over the attribute # Selectively copy options using direct type checks to avoid method calls = { skip_validation: true } .each do |key, value| [key] = case value when Symbol, TrueClass, FalseClass, Numeric, Class, Module, Proc, Method, NilClass value # Immutable, don't dup when Range # Only dup if bounds are mutable strings if value.begin.is_a?(String) || (value.end && value.end.is_a?(String)) Range.new(value.begin.dup, value.end&.dup, value.exclude_end?) else value # Immutable bounds, safe to reuse end when Hash Utils.deep_dup(value) when Array Utils.deep_dup(value) else value # Keep as-is (might be a complex object) end end # Skip validation during deep_dup - options are already validated in original # This prevents infinite recursion when process_options! tries to access collection self.class.new(name, unresolved_type, ).tap do |dup_attr| # Copy already-processed instance variables directly dup_attr.send(:raw=, @raw) dup_attr.send(:validations=, @validations) end end |
#default(register = Lutaml::Model::Config.default_register, instance_object = nil) ⇒ Object
332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 |
# File 'lib/lutaml/model/attribute.rb', line 332 def default(register = Lutaml::Model::Config.default_register, instance_object = nil) if instance_object.nil? @default_cache ||= {} cached = @default_cache[register] return cached if cached result = cast_value(default_value(register, nil), register) if immutable_value?(result) @default_cache[register] = result end result else cast_value(default_value(register, instance_object), register) end end |
#default_set?(register, instance_object = nil) ⇒ Boolean
372 373 374 |
# File 'lib/lutaml/model/attribute.rb', line 372 def default_set?(register, instance_object = nil) !Utils.uninitialized?(default_value(register, instance_object)) end |
#default_type_context ⇒ Object
Performance: Cache default type context lookup
238 239 240 241 242 243 244 |
# File 'lib/lutaml/model/attribute.rb', line 238 def default_type_context @default_type_context ||= begin default_id = Lutaml::Model::Config.default_register ctx = GlobalContext.context(default_id) ctx || GlobalContext.default_context end end |
#default_value(register, instance_object = nil) ⇒ Object
355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 |
# File 'lib/lutaml/model/attribute.rb', line 355 def default_value(register, instance_object = nil) if delegate type(register).attributes(register)[to].default(register, instance_object) elsif [:default].is_a?(Proc) if instance_object instance_object.instance_exec(&[:default]) else [:default].call end elsif .key?(:default) [:default] else Lutaml::Model::UninitializedClass.instance end end |
#delegate ⇒ Object
258 259 260 |
# File 'lib/lutaml/model/attribute.rb', line 258 def delegate @options[:delegate] end |
#derived? ⇒ Boolean
254 255 256 |
# File 'lib/lutaml/model/attribute.rb', line 254 def derived? !method_name.nil? end |
#enum? ⇒ Boolean
328 329 330 |
# File 'lib/lutaml/model/attribute.rb', line 328 def enum? !enum_values.empty? end |
#enum_values ⇒ Object
383 384 385 |
# File 'lib/lutaml/model/attribute.rb', line 383 def enum_values @options.key?(:values) ? @options[:values] : EMPTY_VALUES_ARRAY end |
#execute_validations!(value) ⇒ Object
execute custom validations on the attribute value i.e presence: true, numericality: true, etc
450 451 452 453 454 455 456 457 458 459 460 |
# File 'lib/lutaml/model/attribute.rb', line 450 def execute_validations!(value) return true if Utils.blank?(value) memoization_container = {} errors = Lutaml::Model::Validator.call(value, validations, memoization_container) return if errors.empty? raise Lutaml::Model::ValidationFailedError.new(errors) end |
#extract_register(_context_or_register) ⇒ Object
Extract the Register from the context_or_register argument
209 210 211 212 213 |
# File 'lib/lutaml/model/attribute.rb', line 209 def extract_register(_context_or_register) # Register backward compatibility - now always returns nil # Type resolution uses GlobalContext directly nil end |
#hash_type? ⇒ Boolean
314 315 316 |
# File 'lib/lutaml/model/attribute.rb', line 314 def hash_type? type == Lutaml::Model::Type::Hash end |
#immutable_value?(value) ⇒ Boolean
349 350 351 352 353 |
# File 'lib/lutaml/model/attribute.rb', line 349 def immutable_value?(value) value.nil? || value.is_a?(Numeric) || value.is_a?(String) || value.is_a?(Symbol) || value == true || value == false || value.frozen? end |
#initialize_empty? ⇒ Boolean
273 274 275 |
# File 'lib/lutaml/model/attribute.rb', line 273 def initialize_empty? @options[:initialize_empty] end |
#method_name ⇒ Object
269 270 271 |
# File 'lib/lutaml/model/attribute.rb', line 269 def method_name @options[:method_name] end |
#model_instance?(value) ⇒ Boolean
576 577 578 579 580 |
# File 'lib/lutaml/model/attribute.rb', line 576 def model_instance?(value) return false unless @options[:ref_model_class] value.class.name == @options[:ref_model_class] end |
#normalize_context(context_or_register) ⇒ Object
Normalize register/context to TypeContext for resolution Performance: Optimized to reduce allocations in hot path
217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/lutaml/model/attribute.rb', line 217 def normalize_context(context_or_register) # Fast path for nil - most common case return default_type_context if context_or_register.nil? # Fast path for TypeContext - no conversion needed return context_or_register if context_or_register.is_a?(Lutaml::Model::TypeContext) # Handle Register and Symbol cases context_id = case context_or_register when Lutaml::Model::Register context_or_register.id when Symbol context_or_register else return GlobalContext.default_context end GlobalContext.context(context_id) || GlobalContext.default_context end |
#pattern ⇒ Object
376 377 378 |
# File 'lib/lutaml/model/attribute.rb', line 376 def pattern [:pattern] end |
#polymorphic? ⇒ Boolean
250 251 252 |
# File 'lib/lutaml/model/attribute.rb', line 250 def polymorphic? @options[:polymorphic_class] end |
#process_options! ⇒ Object
676 677 678 679 680 681 |
# File 'lib/lutaml/model/attribute.rb', line 676 def (@options) @raw = !!@options[:raw] @validations = @options[:validations] set_default_for_collection if collection? end |
#raw? ⇒ Boolean
Collection methods are provided by CollectionHandler module
324 325 326 |
# File 'lib/lutaml/model/attribute.rb', line 324 def raw? @raw end |
#reference_key(value) ⇒ Object
567 568 569 570 571 572 573 574 |
# File 'lib/lutaml/model/attribute.rb', line 567 def reference_key(value) return nil unless value return value.map { |item| reference_key(item) } if value.is_a?(Array) return value.public_send(@options[:ref_key_attribute]) if model_instance?(value) value end |
#required_value_set?(value) ⇒ Boolean
291 292 293 294 295 296 297 |
# File 'lib/lutaml/model/attribute.rb', line 291 def required_value_set?(value) return true unless [:required] return false if value.nil? return false if Utils.empty?(value) true end |
#resolver ⇒ Object
Cache the resolver reference — GlobalContext.resolver is a singleton that never changes during the process lifetime. Avoids repeated singleton access overhead on every type() call.
129 130 131 |
# File 'lib/lutaml/model/attribute.rb', line 129 def resolver @resolver ||= GlobalContext.resolver end |
#sequenced_appearance_count(element_order, mapped_name, current_index) ⇒ Object
resolved_collection is provided by CollectionHandler module
649 650 651 652 653 654 655 656 657 658 659 660 661 |
# File 'lib/lutaml/model/attribute.rb', line 649 def sequenced_appearance_count(element_order, mapped_name, current_index) elements = element_order[current_index..] element_count = elements.take_while do |element| element == mapped_name end.count return element_count if element_count.between?(*resolved_collection.minmax) raise Lutaml::Model::ElementCountOutOfRangeError.new( mapped_name, element_count, collection, ) end |
#serializable?(register) ⇒ Boolean
631 632 633 |
# File 'lib/lutaml/model/attribute.rb', line 631 def serializable?(register) type(register) <= Serialize end |
#serializable_type?(register = nil) ⇒ Boolean
Cached version of serializable? check for hot deserialization path Called millions of times in resolve_rule_names_with_type and value_for_rule
637 638 639 640 641 642 643 644 645 |
# File 'lib/lutaml/model/attribute.rb', line 637 def serializable_type?(register = nil) register_key = register || :default return @serializable_type_cache[register_key] if defined?(@serializable_type_cache) && @serializable_type_cache&.key?(register_key) resolved_type = type(register_key) result = resolved_type.is_a?(Class) && resolved_type.include?(Serialize) @serializable_type_cache ||= {} @serializable_type_cache[register_key] = result end |
#serialize(value, format, register, options = {}) ⇒ Object
548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 |
# File 'lib/lutaml/model/attribute.rb', line 548 def serialize(value, format, register, = {}) value ||= build_collection if collection? && initialize_empty? return value if value.nil? || Utils.uninitialized?(value) resolved_type = [:resolved_type] || type(register) = .merge(resolved_type: resolved_type) value = reference_key(value) if unresolved_type == Lutaml::Model::Type::Reference if collection_instance?(value) return serialize_array(value, format, register, ) end if resolved_type <= Serialize return serialize_model(value, format, register, ) end serialize_value(value, format, resolved_type) end |
#setter ⇒ Object
318 319 320 |
# File 'lib/lutaml/model/attribute.rb', line 318 def setter :"#{@name}=" end |
#transform ⇒ Object
265 266 267 |
# File 'lib/lutaml/model/attribute.rb', line 265 def transform @options[:transform] || EMPTY_TRANSFORM_HASH end |
#transform_export_method ⇒ Object
391 392 393 |
# File 'lib/lutaml/model/attribute.rb', line 391 def transform_export_method transform[:export] end |
#transform_import_method ⇒ Object
387 388 389 |
# File 'lib/lutaml/model/attribute.rb', line 387 def transform_import_method transform[:import] end |
#type(context_or_register = nil) ⇒ Object
133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 |
# File 'lib/lutaml/model/attribute.rb', line 133 def type(context_or_register = nil) return if unresolved_type.nil? # Performance: Fast path for nil or default register (most common case during deserialization) if context_or_register.nil? || context_or_register == :default return @cached_type_default ||= begin context = normalize_context(context_or_register) resolver.resolve(unresolved_type, context) end end # Non-default register path: resolve directly without normalize_context # when context_or_register is already a TypeContext or Symbol (common cases) context = case context_or_register when Lutaml::Model::TypeContext context_or_register when Symbol GlobalContext.context(context_or_register) || GlobalContext.default_context when Lutaml::Model::Register GlobalContext.context(context_or_register.id) || GlobalContext.default_context else GlobalContext.default_context end # Performance: Cache per TypeContext identity. # When type substitution replaces a TypeContext, the object_id changes, # so old cache entries are never hit again (safe invalidation). cache_key = context.object_id @type_cache ||= {} cached = @type_cache[cache_key] return cached if cached resolved = resolver.resolve(unresolved_type, context) @type_cache[cache_key] = resolved resolved end |
#type_namespace_class(register = nil) ⇒ Class?
Get namespace class from Type::Value or Model class
725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 |
# File 'lib/lutaml/model/attribute.rb', line 725 def type_namespace_class(register = nil) register_key = register || :default return @type_ns_class_cache[register_key] if defined?(@type_ns_class_cache) && @type_ns_class_cache&.key?(register_key) # Resolve type namespace via type() which uses GlobalContext.resolver resolved_type = type(register_key) result = nil # Check if type is a Type::Value class # Type namespaces are ONLY declared on Type::Value subclasses, # not on Serializable models. Serializable models have element # namespaces, which are handled separately. if resolved_type.is_a?(Class) && resolved_type <= Lutaml::Model::Type::Value result = resolved_type.namespace_class end @type_ns_class_cache ||= {} @type_ns_class_cache[register_key] = result end |
#type_namespace_prefix(register = nil) ⇒ String?
Get namespace prefix from type
764 765 766 |
# File 'lib/lutaml/model/attribute.rb', line 764 def type_namespace_prefix(register = nil) type_namespace_class(register)&.prefix_default end |
#type_namespace_uri(register = nil) ⇒ String?
Get namespace URI from type
750 751 752 753 754 755 756 757 |
# File 'lib/lutaml/model/attribute.rb', line 750 def type_namespace_uri(register = nil) register_key = register || :default return @type_ns_uri_cache[register_key] if defined?(@type_ns_uri_cache) && @type_ns_uri_cache&.key?(register_key) result = type_namespace_class(register_key)&.uri @type_ns_uri_cache ||= {} @type_ns_uri_cache[register_key] = result end |
#type_with_namespace(register, namespace_uri = nil) ⇒ Class?
Get type with namespace-aware resolution.
Uses the register’s resolve_in_namespace method for version-aware type resolution. Falls back to standard resolution if no namespace or register is provided.
180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 |
# File 'lib/lutaml/model/attribute.rb', line 180 def type_with_namespace(register, namespace_uri = nil) return type(register) unless register && namespace_uri return if unresolved_type.nil? # Performance: Memoize type resolution per (register, namespace_uri) pair # This avoids repeated resolve_in_namespace calls for the same attribute @type_with_namespace_cache ||= {} cache_key = [register, namespace_uri] return @type_with_namespace_cache[cache_key] if @type_with_namespace_cache.key?(cache_key) # Resolve register from symbol if needed actual_register = if register.is_a?(Symbol) Lutaml::Model::GlobalRegister.lookup(register) else register end # If we don't have an actual Register object, fall back to standard resolution result = if actual_register.is_a?(Lutaml::Model::Register) actual_register.resolve_in_namespace(unresolved_type, namespace_uri) end result ||= type(register) @type_with_namespace_cache[cache_key] = result result end |
#unresolved_type ⇒ Object
246 247 248 |
# File 'lib/lutaml/model/attribute.rb', line 246 def unresolved_type @unresolved_type || @type end |
#valid_collection!(value, caller) ⇒ Object
510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 |
# File 'lib/lutaml/model/attribute.rb', line 510 def valid_collection!(value, caller) if collection_instance?(value) && !collection? raise Lutaml::Model::CollectionTrueMissingError.new(name, caller) end return true unless collection? # Allow any value for unbounded collections return true if collection == true unless (Utils.uninitialized?(value) && resolved_collection.min.zero?) || collection_instance?(value) raise Lutaml::Model::CollectionCountOutOfRangeError.new( name, value, collection, ) end return true unless resolved_collection.is_a?(Range) if resolved_collection.is_a?(Range) && resolved_collection.end.infinite? if value.size < resolved_collection.begin raise Lutaml::Model::CollectionCountOutOfRangeError.new( name, value, collection, ) end elsif resolved_collection.is_a?(Range) && !resolved_collection.cover?(value.size) raise Lutaml::Model::CollectionCountOutOfRangeError.new( name, value, collection, ) end end |
#valid_pattern!(value, resolved_type) ⇒ Object
413 414 415 416 417 418 419 420 421 422 |
# File 'lib/lutaml/model/attribute.rb', line 413 def valid_pattern!(value, resolved_type) return true unless resolved_type == Lutaml::Model::Type::String return true unless pattern unless pattern.match?(value) raise Lutaml::Model::PatternNotMatchedError.new(name, pattern, value) end true end |
#valid_value!(value) ⇒ Object
395 396 397 398 399 400 401 402 403 404 405 |
# File 'lib/lutaml/model/attribute.rb', line 395 def valid_value!(value) return true if value.nil? && singular? return true unless enum? return true if Utils.uninitialized?(value) unless valid_value?(value) raise Lutaml::Model::InvalidValueError.new(name, value, enum_values) end true end |
#valid_value?(value) ⇒ Boolean
407 408 409 410 411 |
# File 'lib/lutaml/model/attribute.rb', line 407 def valid_value?(value) return true unless [:values] [:values].include?(value) end |
#validate_choice_content!(elements) ⇒ Object
663 664 665 666 667 668 |
# File 'lib/lutaml/model/attribute.rb', line 663 def validate_choice_content!(elements) return elements.count unless resolved_collection return 1 if elements.count.between?(*resolved_collection.minmax) elements.each_slice(resolved_collection.max).count end |
#validate_collection_range ⇒ Object
479 480 481 482 483 484 485 486 487 488 489 |
# File 'lib/lutaml/model/attribute.rb', line 479 def validate_collection_range range = @options[:collection] return if range == true return if custom_collection? unless range.is_a?(Range) raise ArgumentError, "Invalid collection range: #{range}" end validate_range!(range) end |
#validate_polymorphic(value, resolved_type) ⇒ Object
462 463 464 465 466 467 468 469 470 471 |
# File 'lib/lutaml/model/attribute.rb', line 462 def validate_polymorphic(value, resolved_type) if value.is_a?(Array) return value.all? do |v| validate_polymorphic!(v, resolved_type) end end return true unless [:polymorphic] valid_polymorphic_type?(value, resolved_type) end |
#validate_polymorphic!(value, resolved_type) ⇒ Object
473 474 475 476 477 |
# File 'lib/lutaml/model/attribute.rb', line 473 def validate_polymorphic!(value, resolved_type) return true if validate_polymorphic(value, resolved_type) raise Lutaml::Model::PolymorphicError.new(value, , resolved_type) end |
#validate_range!(range) ⇒ Object
491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 |
# File 'lib/lutaml/model/attribute.rb', line 491 def validate_range!(range) if range.begin.nil? raise ArgumentError, "Invalid collection range: #{range}. Begin must be specified." end if range.begin.negative? raise ArgumentError, "Invalid collection range: #{range}. " \ "Begin must be non-negative." end if range.end && range.end < range.begin raise ArgumentError, "Invalid collection range: #{range}. " \ "End must be greater than or equal to begin." end end |
#validate_value!(value, register, resolver = nil) ⇒ Object
Check if the value to be assigned is valid for the attribute
Currently there are 2 validations
1. Value should be from the values list if they are defined
e.g values: ["foo", "bar"] is set then any other value for this
attribute will raise `Lutaml::Model::InvalidValueError`
2. Value count should be between the collection range if defined
e.g if collection: 0..5 is set then the value greater then 5
will raise `Lutaml::Model::CollectionCountOutOfRangeError`
434 435 436 437 438 439 440 441 442 443 444 445 446 |
# File 'lib/lutaml/model/attribute.rb', line 434 def validate_value!(value, register, resolver = nil) # Use the default value if the value is nil validate_required!(value) value = resolver&.default if value.nil? resolved_type = type(register) valid_value!(value) && valid_collection!(value, self) && valid_pattern!(value, resolved_type) && validate_polymorphic!(value, resolved_type) && execute_validations!(value) end |