Class: Lutaml::Xml::ProcessingInstructionMapping
- Inherits:
-
Object
- Object
- Lutaml::Xml::ProcessingInstructionMapping
- Defined in:
- lib/lutaml/xml/mapping.rb
Overview
Represents a processing instruction mapping in the XML DSL.
Maps a PI target (e.g., “rfc”) to a model attribute. The attribute should be a Hash where keys are PI parameter names and values are their string values.
Instance Attribute Summary collapse
-
#target ⇒ Object
readonly
Returns the value of attribute target.
-
#to ⇒ Object
readonly
Returns the value of attribute to.
Instance Method Summary collapse
- #dup ⇒ Object
-
#initialize(target, to) ⇒ ProcessingInstructionMapping
constructor
A new instance of ProcessingInstructionMapping.
Constructor Details
#initialize(target, to) ⇒ ProcessingInstructionMapping
Returns a new instance of ProcessingInstructionMapping.
1471 1472 1473 1474 |
# File 'lib/lutaml/xml/mapping.rb', line 1471 def initialize(target, to) @target = target @to = to end |
Instance Attribute Details
#target ⇒ Object (readonly)
Returns the value of attribute target.
1469 1470 1471 |
# File 'lib/lutaml/xml/mapping.rb', line 1469 def target @target end |
#to ⇒ Object (readonly)
Returns the value of attribute to.
1469 1470 1471 |
# File 'lib/lutaml/xml/mapping.rb', line 1469 def to @to end |
Instance Method Details
#dup ⇒ Object
1476 1477 1478 |
# File 'lib/lutaml/xml/mapping.rb', line 1476 def dup self.class.new(@target.dup, @to) end |