Class: Lutaml::Xml::ProcessingInstructionMapping

Inherits:
Object
  • Object
show all
Defined in:
lib/lutaml/xml/mapping.rb

Overview

Represents a processing instruction mapping in the XML DSL.

Maps a PI target (e.g., “rfc”) to a model attribute. The attribute should be a Hash where keys are PI parameter names and values are their string values.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(target, to) ⇒ ProcessingInstructionMapping

Returns a new instance of ProcessingInstructionMapping.



1469
1470
1471
1472
# File 'lib/lutaml/xml/mapping.rb', line 1469

def initialize(target, to)
  @target = target
  @to = to
end

Instance Attribute Details

#targetObject (readonly)

Returns the value of attribute target.



1467
1468
1469
# File 'lib/lutaml/xml/mapping.rb', line 1467

def target
  @target
end

#toObject (readonly)

Returns the value of attribute to.



1467
1468
1469
# File 'lib/lutaml/xml/mapping.rb', line 1467

def to
  @to
end

Instance Method Details

#dupObject



1474
1475
1476
# File 'lib/lutaml/xml/mapping.rb', line 1474

def dup
  self.class.new(@target.dup, @to)
end