Class: Lutaml::Xml::ProcessingInstructionMapping
- Inherits:
-
Object
- Object
- Lutaml::Xml::ProcessingInstructionMapping
- Defined in:
- lib/lutaml/xml/mapping.rb
Overview
Represents a processing instruction mapping in the XML DSL.
Maps a PI target (e.g., “rfc”) to a model attribute. The attribute should be a Hash where keys are PI parameter names and values are their string values.
Instance Attribute Summary collapse
-
#target ⇒ Object
readonly
Returns the value of attribute target.
-
#to ⇒ Object
readonly
Returns the value of attribute to.
Instance Method Summary collapse
- #dup ⇒ Object
-
#initialize(target, to) ⇒ ProcessingInstructionMapping
constructor
A new instance of ProcessingInstructionMapping.
Constructor Details
#initialize(target, to) ⇒ ProcessingInstructionMapping
Returns a new instance of ProcessingInstructionMapping.
1460 1461 1462 1463 |
# File 'lib/lutaml/xml/mapping.rb', line 1460 def initialize(target, to) @target = target @to = to end |
Instance Attribute Details
#target ⇒ Object (readonly)
Returns the value of attribute target.
1458 1459 1460 |
# File 'lib/lutaml/xml/mapping.rb', line 1458 def target @target end |
#to ⇒ Object (readonly)
Returns the value of attribute to.
1458 1459 1460 |
# File 'lib/lutaml/xml/mapping.rb', line 1458 def to @to end |
Instance Method Details
#dup ⇒ Object
1465 1466 1467 |
# File 'lib/lutaml/xml/mapping.rb', line 1465 def dup self.class.new(@target.dup, @to) end |